@@ -1,6 +1,6 @@
 #' Dominic Schuhmacher's idea
 #'
-#' $Revision: 1.15 $ $Date: 2016/03/02 09:41:32 $
+#' $Revision: 1.16 $ $Date: 2017/01/07 09:24:04 $
 #'
 
 clickppp <- local({
@@ -39,8 +39,8 @@ clickppp <- local({
     
     ftypes <- factor(types, levels=types)
     #' input points of type 1 
-    X <- getem(ftypes[1], instructions, n=n, win=win, add=add, ..., pch=1)
-    X <- X %mark% ftypes[1]
+    X <- getem(ftypes[1L], instructions, n=n, win=win, add=add, ..., pch=1)
+    X <- X %mark% ftypes[1L]
     #' input points of types 2, 3, ... in turn
     naughty <- FALSE
     for(i in 2:length(types)) {
@@ -79,38 +79,7 @@ clickppp <- local({
 })
 
 
-spatstatLocator <- function(n, type=c("p","l","o","n"), ...) {
-  #' remedy for failure of locator(type="p") in RStudio
-  if(!identical(TRUE, dev.capabilities()$locator))
-    stop("Sorry, this graphics device does not support the locator() function")
-  # validate
-  type <- match.arg(type)
-  do.points <- type %in% c("p","o")
-  do.lines <- type %in% c("l","o")
-  argh <- list(...)
-  pointsArgs <- c("cex", "col", "pch", "fg", "bg")
-  segmentArgs <- graphicsPars("lines")
-  # go
-  res <- list(x=numeric(0), y = numeric(0))
-  i <- 1
-  if(missing(n)) n <- Inf
-  while(i<=n){
-    tmp <- locator(n=1)
-    if(is.null(tmp)) return(res)
-    if(do.points)
-      do.call.matched(points.default, append(tmp, argh), extrargs=pointsArgs)
-    res$x <- c(res$x,tmp$x)
-    res$y <- c(res$y,tmp$y)
-    if(do.lines && i > 1) {
-      xy <- with(res, list(x0=x[i-1], y0=y[i-1], x1=x[i], y1=y[i]))
-      do.call.matched(segments, append(xy, argh), extrargs=segmentArgs)
-    }
-    i <- i+1
-  }
-  return(res)
-}
-  
 clickdist <- function() {
   a <- spatstatLocator(2)
-  return(pairdist(a)[1,2])
+  return(pairdist(a)[1L,2L])
 }
