@@ -3,7 +3,7 @@
 #
 #  leverage and influence
 #
-#  $Revision: 1.114 $ $Date: 2018/05/24 08:08:31 $
+#  $Revision: 1.116 $ $Date: 2019/02/15 01:31:36 $
 #
 
 leverage <- function(model, ...) {
@@ -78,7 +78,7 @@ leverage.ppmInfluence <- function(model, ...) { model$leverage }
 influence.ppmInfluence <- function(model, ...) { model$influence }
 dfbetas.ppmInfluence <- function(model, ...) { model$dfbetas }
 
-avenndist <- function(X) mean(nndist(X))
+avenndist <- function(X) mean(nndist(unique(X)))
 
 ## ...............  main workhorse ....................................
 
@@ -926,7 +926,8 @@ plot.leverage.ppm <- function(x, ...,
   if(!multiplot && inherits(smo, "imlist")) {
     ave <- ave * length(smo)
     smo <- do.call(harmonise.im, unname(smo))
-    smo <- Reduce("+", smo)
+    ##    smo <- Reduce("+", smo)
+    smo <- im.apply(smo, sum, check=FALSE)
     defaultmain <- c(defaultmain, "(sum over all types of point)")
   }
   args.contour <- resolve.defaults(args.cut, list(levels=ave))
@@ -974,7 +975,8 @@ contour.leverage.ppm <- function(x, ...,
   if(!multiplot && inherits(smo, "imlist")) {
     ave <- ave * length(smo)
     smo <- do.call(harmonise.im, unname(smo))
-    smo <- Reduce("+", smo)
+    ##    smo <- Reduce("+", smo)
+    smo <- im.apply(smo, sum, check=FALSE)
     defaultmain <- c(defaultmain, "(sum over all types of point)")
   }
   
