@@ -1,7 +1,7 @@
 #
 #   plot.im.R
 #
-#  $Revision: 1.127 $   $Date: 2019/03/08 10:13:03 $
+#  $Revision: 1.130 $   $Date: 2019/07/03 03:33:42 $
 #
 #  Plotting code for pixel images
 #
@@ -98,8 +98,8 @@ plot.im <- local({
     if(box)
       rect(xr[1], yr[1], xr[2], yr[2])
     if(axes) {
-      px <- pretty(xr)
-      py <- pretty(yr)
+      px <- prettyinside(xr)
+      py <- prettyinside(yr)
       do.call.plotfun(graphics::axis,
                       resolve.defaults(
                                        list(side=1, at=px), 
@@ -169,8 +169,8 @@ plot.im <- local({
                      ribbon=show.all, show.all=!add,
                      ribside=c("right", "left", "bottom", "top"),
                      ribsep=0.15, ribwid=0.05, ribn=1024,
-                     ribscale=1, ribargs=list(), colargs=list(),
-                     useRaster=NULL, workaround=FALSE,
+                     ribscale=1, ribargs=list(), riblab=NULL, colargs=list(),
+                     useRaster=NULL, workaround=FALSE, zap=1,
                      do.plot=TRUE) {
     if(missing(main)) main <- short.deparse(substitute(x))
     verifyclass(x, "im")
@@ -331,7 +331,7 @@ plot.im <- local({
                vrange <- range(imagebreaks)
                col <- s$outputs
              } 
-             trivial <- (diff(vrange) <= .Machine$double.eps)
+             trivial <- (diff(vrange) <= zap * .Machine$double.eps)
              #' ribbonvalues: domain of colour map (pixel values)
              #' ribbonrange: (min, max) of pixel values in image
              #' nominalrange: range of values shown on ribbon 
@@ -723,6 +723,11 @@ plot.im <- local({
                       resolve.defaults(ribargs, axisargs, dotargs, posargs),
                       extrargs=graphicsPars("axis"))
     }
+    if(!is.null(riblab)) {
+      riblabel <- if(is.list(riblab)) riblab else list(text=riblab)
+      riblabel$side <- rib.iside
+      do.call(mtext, riblabel)
+    }
     #
     return(invisible(output.colmap))
   }
