@@ -1,12 +1,15 @@
-#
-#  rhohat.R
-#
-#  $Revision: 1.79 $  $Date: 2018/09/23 10:33:25 $
-#
-#  Non-parametric estimation of a transformation rho(z) determining
-#  the intensity function lambda(u) of a point process in terms of a
-#  spatial covariate Z(u) through lambda(u) = rho(Z(u)).
-#  More generally allows offsets etc.
+#'
+#'  rhohat.R
+#'
+#'  $Revision: 1.84 $  $Date: 2019/09/09 10:07:59 $
+#'
+#'  Non-parametric estimation of a transformation rho(z) determining
+#'  the intensity function lambda(u) of a point process in terms of a
+#'  spatial covariate Z(u) through lambda(u) = rho(Z(u)).
+#'  More generally allows offsets etc.
+
+#' Copyright (c) Adrian Baddeley 2015-2019
+#' GNU Public Licence GPL >= 2.0
 
 rhohat <- function(object, covariate, ...) {
   UseMethod("rhohat")
@@ -17,7 +20,7 @@ rhohat.ppp <- rhohat.quad <-
            baseline=NULL, weights=NULL,
            method=c("ratio", "reweight", "transform"),
            horvitz=FALSE,
-           smoother=c("kernel", "local"),
+           smoother=c("kernel", "local", "decreasing", "increasing"),
            subset=NULL,
            dimyx=NULL, eps=NULL,
            n=512, bw="nrd0", adjust=1, from=NULL, to=NULL, 
@@ -79,7 +82,7 @@ rhohat.ppm <- function(object, covariate, ...,
                        weights=NULL,
                        method=c("ratio", "reweight", "transform"),
                        horvitz=FALSE,
-                       smoother=c("kernel", "local"),
+                       smoother=c("kernel", "local", "decreasing", "increasing"),
                        subset=NULL,
                        dimyx=NULL, eps=NULL,
                        n=512, bw="nrd0", adjust=1, from=NULL, to=NULL, 
@@ -140,7 +143,7 @@ rhohat.lpp <- rhohat.lppm <-
            weights=NULL,
            method=c("ratio", "reweight", "transform"),
            horvitz=FALSE,
-           smoother=c("kernel", "local"),
+           smoother=c("kernel", "local", "decreasing", "increasing"),
            subset=NULL,
            nd=1000, eps=NULL, random=TRUE, 
            n=512, bw="nrd0", adjust=1, from=NULL, to=NULL, 
@@ -212,7 +215,7 @@ rhohatEngine <- function(model, covariate,
                          weights=NULL,
                          method=c("ratio", "reweight", "transform"),
                          horvitz=FALSE,
-                         smoother=c("kernel", "local"),
+                         smoother=c("kernel", "local", "decreasing", "increasing"),
                          resolution=list(), 
                          n=512, bw="nrd0", adjust=1, from=NULL, to=NULL, 
                          bwref=bw, covname, covunits=NULL, confidence=0.95,
@@ -304,7 +307,7 @@ rhohatCalc <- local({
                          weights=NULL, lambdaX,
                          method=c("ratio", "reweight", "transform"),
                          horvitz=FALSE, 
-                         smoother=c("kernel", "local"),
+                         smoother=c("kernel", "local", "decreasing", "increasing"),
                          n=512, bw="nrd0", adjust=1, from=NULL, to=NULL, 
                          bwref=bw, covname, confidence=0.95,
                          positiveCI=(smoother == "local"),
@@ -315,7 +318,7 @@ rhohatCalc <- local({
     smoother <- match.arg(smoother)
     ## check availability of locfit package
     if(smoother == "local" && !requireNamespace("locfit", quietly=TRUE)) {
-      warning(paste("In", paste(dQuote(callstring), ":", sep=""),
+      warning(paste("In", paste0(dQuote(callstring), ":"),
                     "package", sQuote("locfit"), "is not available;",
                     "unable to perform local likelihood smoothing;",
                     "using kernel smoothing instead"),
@@ -342,14 +345,18 @@ rhohatCalc <- local({
     if(is.null(from)) from <- Zrange[1] 
     if(is.null(to))   to   <- Zrange[2]
     if(from > Zrange[1] || to < Zrange[2])
-      stop("Interval [from, to] = ", prange(c(from,to)), 
-           "does not contain the range of data values =", prange(Zrange))
+      stop(paste("In", paste0(dQuote(callstring), ":"),
+                 "interval [from, to] =", prange(c(from,to)), 
+                 "does not contain the range of data values =",
+                 prange(Zrange)),
+           call.=FALSE)
     ## critical constant for CI's
     crit <- qnorm((1+confidence)/2)
     percentage <- paste(round(100 * confidence), "%%", sep="")
     CIblurb <- paste("pointwise", percentage, "confidence interval")
-    ## estimate densities   
-    if(smoother == "kernel") {
+    ## estimate densities
+    switch(smoother,
+    kernel = {
       ## ............... kernel smoothing ......................
       ## reference density (normalised) for calculation
       ghat <- density(Zvalues,weights=if(horvitz) NULL else lambda/sum(lambda),
@@ -437,7 +444,8 @@ rhohatCalc <- local({
           hi <- pmin(hi, yyy/(1-confidence))
         }
       }
-    } else {
+    },
+    local = {
       ## .................. local likelihood smoothing .......................
       xlim <- c(from, to)
       xxx <- seq(from, to, length=n)
@@ -499,32 +507,80 @@ rhohatCalc <- local({
         hi <- yyy * (1 + crit * sdlog)
         lo <- yyy * (1 - crit * sdlog)
       }
-    }
+    },
+    increasing = ,
+    decreasing = {
+      ## .................. nonparametric maximum likelihood ............
+      if(is.null(weights)) weights <- rep(1, length(ZX))
+      #' observed (sorted)
+      oX <- order(ZX)
+      ZX <- ZX[oX]
+      weights <- weights[oX]
+      #' reference CDF
+      G <- ewcdf(Zvalues, lambda)
+      #' reference denominator ('area') at each observed value
+      if(smoother == "decreasing") {
+        areas <- denom * G(ZX)
+      } else {
+        areas <- denom * (1 - G(rev(ZX)))
+        weights <- rev(weights)
+      }
+      #' maximum upper sets algorithm
+      rho <- numeric(0)
+      darea <- diff(c(0, areas))
+      dcount <- weights
+      while(length(darea) > 0) {
+        u <- cumsum(dcount)/cumsum(darea)
+        if(any(bad <- !is.finite(u))) # divide by zero etc
+          u[bad] <- max(u[!bad], 0)
+        k <- which.max(u)
+        rho <- c(rho, rep(u[k], k))
+        darea <- darea[-(1:k)]
+        dcount <- dcount[-(1:k)]
+      }
+      rho <- c(rho, 0)
+      if(smoother == "increasing") rho <- rev(rho)
+      #' compute as a stepfun
+      rhofun <- stepfun(x = ZX, y=rho, right=TRUE, f=1)
+      #' evaluate on a grid
+      xlim <- c(from, to)
+      xxx <- seq(from, to, length=n)
+      yyy <- rhofun(xxx)
+      #'
+      vvv <- hi <- lo <- NULL
+      savestuff$rhofun <- rhofun
+    })
     ## pack into fv object
-    df <- data.frame(xxx=xxx, rho=yyy, var=vvv, hi=hi, lo=lo)
+    df <- data.frame(xxx=xxx, rho=yyy)
     names(df)[1] <- covname
-    desc <- c(paste("covariate", covname),
-              "Estimated intensity",
-              vvvname,
-              paste("Upper limit of", CIblurb),
-              paste("Lower limit of", CIblurb))
+    desc <- c(paste("covariate", covname), "Estimated intensity")
+    labl <- c(covname, paste("hat(%s)", paren(covname), sep=""))
+    if(did.variance <- !is.null(vvv)) {
+      df <- cbind(df, data.frame(var=vvv, hi=hi, lo=lo))
+      desc <- c(desc,
+                vvvname,
+                paste("Upper limit of", CIblurb),
+                paste("Lower limit of", CIblurb))
+      labl <- c(labl,
+                vvvlabel,
+                paste("%s[hi]", paren(covname), sep=""),
+                paste("%s[lo]", paren(covname), sep=""))
+    }
     rslt <- fv(df,
                argu=covname,
                ylab=substitute(rho(X), list(X=as.name(covname))),
                valu="rho",
                fmla= as.formula(paste(". ~ ", covname)),
                alim=range(ZX),
-               labl=c(covname,
-                 paste("hat(%s)", paren(covname), sep=""),
-                 vvvlabel,
-                 paste("%s[hi]", paren(covname), sep=""),
-                 paste("%s[lo]", paren(covname), sep="")),
+               labl=labl,
                desc=desc,
                unitname=covunits,
                fname="rho",
                yexp=substitute(rho(X), list(X=as.name(covname))))
-    fvnames(rslt, ".")  <- c("rho", "hi", "lo")
-    fvnames(rslt, ".s") <- c("hi", "lo")
+    if(did.variance) {
+      fvnames(rslt, ".")  <- c("rho", "hi", "lo")
+      fvnames(rslt, ".s") <- c("hi", "lo")
+    } else fvnames(rslt, ".")  <- "rho"
     ## pack up
     class(rslt) <- c("rhohat", class(rslt))
     ## add info
@@ -560,7 +616,9 @@ print.rhohat <- function(x, ...) {
            splat("Function values are relative to fitted model")
            print(s$modelcall)
          })
+  NPMLE <- s$smoother %in% c("increasing", "decreasing")
   cat("Estimation method: ")
+  if(NPMLE) splat("nonparametric maximum likelihood") else
   switch(s$method,
          ratio={
            splat("ratio of fixed-bandwidth kernel smoothers")
@@ -583,13 +641,18 @@ print.rhohat <- function(x, ...) {
            splat("\tActual smoothing bandwidth sigma = ",
                  signif(s$sigma,5))
          },
-         local ={ splat("Local likelihood density estimator") }
+         local = splat("Local likelihood density estimator"),
+         increasing = splat("Increasing function of covariate"),
+         decreasing = splat("Decreasing function of covariate"),
+         splat("UNKNOWN")
          )
-  positiveCI <- s$positiveCI %orifnull% (s$smoother == "local")
-  confidence <- s$confidence %orifnull% 0.95
-  splat("Pointwise", paste0(100 * confidence, "%"),
-        "confidence bands for rho(x)\n\t based on asymptotic variance of",
-        if(positiveCI) "log(rhohat(x))" else "rhohat(x)")
+  if(!NPMLE) {
+    positiveCI <- s$positiveCI %orifnull% (s$smoother == "local")
+    confidence <- s$confidence %orifnull% 0.95
+    splat("Pointwise", paste0(100 * confidence, "%"),
+          "confidence bands for rho(x)\n\t based on asymptotic variance of",
+          if(positiveCI) "log(rhohat(x))" else "rhohat(x)")
+  }
   splat("Call:", s$callstring)
   cat("\n")
   NextMethod("print")
@@ -600,9 +663,11 @@ plot.rhohat <- function(x, ..., do.rug=TRUE) {
   s <- attr(x, "stuff")
   covname <- s$covname
   asked.rug <- !missing(do.rug) && identical(rug, TRUE)
+  snam <- intersect(c("hi", "lo"), names(x))
+  if(length(snam) == 0) snam <- NULL
   out <- do.call(plot.fv,
                  resolve.defaults(list(x=x), list(...),
-                                  list(main=xname, shade=c("hi", "lo"))))
+                                  list(main=xname, shade=snam)))
   if(identical(list(...)$limitsonly, TRUE))
     return(out)
   if(do.rug) {
@@ -646,6 +711,10 @@ predict.rhohat <- function(object, ..., relative=FALSE,
   # extract info
   s <- attr(object, "stuff")
   reference <- s$reference
+  #' check availability
+  if((what %in% c("lo", "hi", "se")) && !("hi" %in% names(object)))
+    stop("Standard error and confidence bands are not available in this object",
+         call.=FALSE)
   # convert to (linearly interpolated) function 
   x <- with(object, .x)
   y <- if(what == "se") sqrt(object[["var"]]) else object[[what]]
