@@ -7,7 +7,10 @@
 
   poly2imA     pixel value = area of intersection between pixel and polygon
 
-  $Revision: 1.7 $ $Date: 2012/04/06 07:33:43 $
+  $Revision: 1.10 $ $Date: 2018/12/18 02:43:11 $
+
+  Copyright (C) Adrian Baddeley, Ege Rubak and Rolf Turner 2001-2018
+  Licence: GNU Public Licence >= 2
 
 */
 #undef DEBUG
@@ -18,8 +21,6 @@
 
 #include "chunkloop.h"
 
-#define OUT(I,J) out[I + (J) * Ny]
-
 void 
 poly2imI(xp, yp, np, nx, ny, out) 
      double *xp, *yp; /* polygon vertices, anticlockwise, CLOSED  */
@@ -28,7 +29,7 @@ poly2imI(xp, yp, np, nx, ny, out)
      int *out;  /* output matrix [ny, nx], byrow=FALSE, initialised to 0 */
 {
   int Np, Nx, Ny, Np1, maxchunk, mstart, mend;
-  int i, j, k, m;
+  int j, k, m;
   double x0, y0, x1, y1, xleft, xright, yleft, yright;
   double dx, dy, y, slope, intercept;
   int jleft, jright, imax;
@@ -108,7 +109,7 @@ poly2imA(ncol, nrow, xpoly, ypoly, npoly, out, status)
      int *status;
 {
   double *xp, *yp;
-  int nx, ny, np, np1, maxchunk; 
+  int nx, ny, nxy, np, np1, maxchunk; 
   int i, j, k;
   double xcur, ycur, xnext, ynext, xleft, yleft, xright, yright;
   int sgn, jmin, jmax, imin, imax;
@@ -125,9 +126,9 @@ poly2imA(ncol, nrow, xpoly, ypoly, npoly, out, status)
   *status = 0;
 
   /* initialise output array */
-  for(i = 0; i < ny; i++)
-    for(j = 0; j < nx; j++)
-      out[j + ny * i] = 0;
+  nxy = nx * ny;
+  for(k = 0; k < nxy; k++) 
+    out[k] = 0;
 
   /* ............ loop over polygon edges ...................*/
   np1 = np - 1;
@@ -247,7 +248,7 @@ poly2imA(ncol, nrow, xpoly, ypoly, npoly, out, status)
 #ifdef DEBUG
 		Rprintf( "\tIncrementing area by %lf\n", sgn * area);
 #endif
-		out[j + ny * i] += sgn * area;
+		out[i + ny * j] += sgn * area;
 	      }
 	    }
 	    /* second part */
@@ -321,7 +322,7 @@ poly2imA(ncol, nrow, xpoly, ypoly, npoly, out, status)
 #ifdef DEBUG
 	      Rprintf( "\tIncrementing area by %lf\n", sgn * area);
 #endif
-	      out[j + ny * i] += sgn * area;
+	      out[i + ny * j] += sgn * area;
 	    }
 	    /* ............ end of loop over pixels within column ......... */
 	  }
